//--------------------------------------------------------------------------------------
// Exercise04.cpp
// Direct3D 10 Shader Model 4.0 Workshop
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------
#include "dxut.h"
#include "DXUTCamera.h"
#include "SDKmisc.h"

struct TRI_VERTEX
{
	D3DXVECTOR3 Position;
	D3DXVECTOR3 Normal;
	D3DXVECTOR2 Tex;			
};

#define NUM_VERTICES (6)
#define IMPLEMENTATION_STEP (1)
#define MAX_NUM_ITERATIONS (5)
#define DEFAULT_NUM_ITERATIONS (3)

//--------------------------------------------------------------------------------------
// DXUT Specific variables
//--------------------------------------------------------------------------------------
CModelViewerCamera                  g_Camera;               // A model viewing camera

//--------------------------------------------------------------------------------------
// Mesh Specific Variables
//--------------------------------------------------------------------------------------
ID3D10InputLayout*                  g_pVertexLayout = NULL;
ID3D10Buffer*                       g_pVB = NULL;
UINT                                g_SizeVB = 0;
UINT                                g_VertStride = 0;
ID3D10ShaderResourceView*           g_pMeshTexRV = NULL;

//--------------------------------------------------------------------------------------
// Effect variables
//--------------------------------------------------------------------------------------
ID3D10Effect*                       g_pEffect10 = NULL;
ID3D10EffectTechnique*              g_pRenderTextured = NULL;
ID3D10EffectMatrixVariable*         g_pmWorldViewProj = NULL;
ID3D10EffectMatrixVariable*         g_pmWorldView = NULL;
ID3D10EffectMatrixVariable*         g_pmWorld = NULL;
ID3D10EffectMatrixVariable*         g_pmProj = NULL;
ID3D10EffectVectorVariable*         g_pViewSpaceLightDir = NULL;
ID3D10EffectShaderResourceVariable* g_pDiffuseTex = NULL;

ID3D10EffectTechnique*              g_pShowNormals = NULL;
bool                                g_bShowNormals = true;

#if (IMPLEMENTATION_STEP > 0)
// Generate flag, set to true if geometry needs to be created
bool g_bGenerate = true; 

// Geometry Stream Out Buffers
ID3D10Buffer*                       g_pStreamTo = NULL;
ID3D10Buffer*                       g_pDrawFrom = NULL;

// Geometry Shader Technique which writes to stream out
ID3D10EffectTechnique*              g_pSmooth = NULL;

// recursion depth
UINT                                g_NumIterations = DEFAULT_NUM_ITERATIONS;
#endif // (IMPLEMENTATION_STEP > 0)

//--------------------------------------------------------------------------------------
// Forward declarations 
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext );
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext );
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing,
						 void* pUserContext );
bool CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType,
									  DXGI_FORMAT BackBufferFormat, bool bWindowed, void* pUserContext );
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc,
									 void* pUserContext );
HRESULT CALLBACK OnD3D10SwapChainResized( ID3D10Device* pd3dDevice, IDXGISwapChain* pSwapChain,
										 const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext );
void CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext );
void CALLBACK OnD3D10DestroyDevice( void* pUserContext );

void CALLBACK KeyboardProc( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext );

HRESULT LoadMesh( ID3D10Device* pd3dDevice );


//--------------------------------------------------------------------------------------
// Entry point to the program. Initializes everything and goes into a message processing 
// loop. Idle time is used to render the scene.
//--------------------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE, HINSTANCE, LPSTR, int )
{
	// DXUT will create and use the best device 
	// that is available on the system depending on which D3D callbacks are set below

	// Set DXUT callbacks
	DXUTSetCallbackDeviceChanging( ModifyDeviceSettings );
	DXUTSetCallbackMsgProc( MsgProc );
	DXUTSetCallbackFrameMove( OnFrameMove );
	DXUTSetCallbackD3D10DeviceAcceptable( IsD3D10DeviceAcceptable );
	DXUTSetCallbackD3D10DeviceCreated( OnD3D10CreateDevice );
	DXUTSetCallbackD3D10SwapChainResized( OnD3D10SwapChainResized );
	DXUTSetCallbackD3D10DeviceDestroyed( OnD3D10DestroyDevice );
	DXUTSetCallbackD3D10FrameRender( OnD3D10FrameRender );
	DXUTSetCallbackKeyboard( KeyboardProc );

	DXUTInit( true, true, NULL ); // Parse the command line, show msgboxes on error, no extra command line params
	DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
	DXUTCreateWindow( L"Tutorial11: Geometry Shader Procedural Tree" );
	DXUTCreateDevice( true, 800, 600 );
	DXUTMainLoop(); // Enter into the DXUT render loop

	return DXUTGetExitCode();
}


//--------------------------------------------------------------------------------------
// Called right before creating a D3D9 or D3D10 device, allowing the app to modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext )
{
	return true;
}


//--------------------------------------------------------------------------------------
// Handle updates to the scene.  This is called regardless of which D3D API is used
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext )
{
	// Update the camera's position based on user input 
	g_Camera.FrameMove( fElapsedTime );
}


//--------------------------------------------------------------------------------------
// Handle messages to the application
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing,
						 void* pUserContext )
{

	// Pass all remaining windows messages to camera so it can respond to user input
	g_Camera.HandleMessages( hWnd, uMsg, wParam, lParam );

	return 0;
}

//--------------------------------------------------------------------------------------
// Handle key presses
//--------------------------------------------------------------------------------------
void CALLBACK KeyboardProc( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext )
{
	if (bKeyDown) {
		switch(nChar) {

#if (IMPLEMENTATION_STEP > 0)

case 0x41: // Virtual Key A
	if (g_NumIterations < MAX_NUM_ITERATIONS) {
		g_NumIterations += 1;
		g_bGenerate = true;
	}
	break;

case 0x42: // Virtual Key B
	if (g_NumIterations > 0) {
		g_NumIterations -= 1;
		g_bGenerate = true;
	}
	break;

#endif // (IMPLEMENTATION_STEP > 0)

case 0x4E: // Virtual Key N
	g_bShowNormals = !g_bShowNormals;
	break;
		}

	}
}

//--------------------------------------------------------------------------------------
// Reject any D3D10 devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType,
									  DXGI_FORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
	return true;
}


//--------------------------------------------------------------------------------------
// Create any D3D10 resources that aren't dependant on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc,
									 void* pUserContext )
{
	HRESULT hr;

	V_RETURN( CDXUTDirectionWidget::StaticOnD3D10CreateDevice( pd3dDevice ) );

	// Read the D3DX effect file
	WCHAR str[MAX_PATH];
	DWORD dwShaderFlags = D3D10_SHADER_ENABLE_BACKWARDS_COMPATIBILITY;
#if defined( DEBUG ) || defined( _DEBUG )
	// Set the D3D10_SHADER_DEBUG flag to embed debug information in the shaders.
	// Setting this flag improves the shader debugging experience, but still allows 
	// the shaders to be optimized and to run exactly the way they will run in 
	// the release configuration of this program.
	dwShaderFlags |= D3D10_SHADER_DEBUG;
#endif

#if (IMPLEMENTATION_STEP > 0)
	V_RETURN( DXUTFindDXSDKMediaFileCch( str, MAX_PATH, L"..\\src\\Tutorial12_Recursive.fx" ) );
#else
	V_RETURN( DXUTFindDXSDKMediaFileCch( str, MAX_PATH, L"..\\src\\Tutorial12.fx" ) );
#endif

	V_RETURN( D3DX10CreateEffectFromFile( str, NULL, NULL, "fx_4_0", dwShaderFlags, 0, pd3dDevice, NULL,
		NULL, &g_pEffect10, NULL, NULL ) );

	// Obtain the technique handles
	g_pRenderTextured = g_pEffect10->GetTechniqueByName( "RenderTextured" );
	g_pShowNormals = g_pEffect10->GetTechniqueByName( "ShowNormals" );

	// Obtain the parameter handles
	g_pmWorldViewProj = g_pEffect10->GetVariableByName( "g_mWorldViewProj" )->AsMatrix();
	g_pmWorldView = g_pEffect10->GetVariableByName( "g_mWorldView" )->AsMatrix();
	g_pmWorld = g_pEffect10->GetVariableByName( "g_mWorld" )->AsMatrix();
	g_pmProj = g_pEffect10->GetVariableByName( "g_mProj" )->AsMatrix();
	g_pViewSpaceLightDir = g_pEffect10->GetVariableByName( "g_ViewSpaceLightDir" )->AsVector();
	g_pDiffuseTex = g_pEffect10->GetVariableByName( "g_txDiffuse" )->AsShaderResource();

#if (IMPLEMENTATION_STEP > 0)
	g_pSmooth = g_pEffect10->GetTechniqueByName( "Smooth" );
#endif // (IMPLEMENTATION_STEP > 0)

	// Define our vertex data layout
	const D3D10_INPUT_ELEMENT_DESC layout[] =
	{
		{ "POS",      0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
		{ "NORMAL",   0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
		{ "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT,    0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	};
	D3D10_PASS_DESC PassDesc;
	g_pRenderTextured->GetPassByIndex( 0 )->GetDesc( &PassDesc );
	V_RETURN( pd3dDevice->CreateInputLayout( layout, 3, PassDesc.pIAInputSignature,
		PassDesc.IAInputSignatureSize, &g_pVertexLayout ) );

	// Load our IB and VB with mesh data
	V_RETURN( LoadMesh( pd3dDevice ) );

	// Setup the camera's view parameters
	D3DXVECTOR3 vecEye( 0.0f, 0.0f, -7.0f );
	D3DXVECTOR3 vecAt( 0.0f,0.0f,0.0f );
	g_Camera.SetViewParams( &vecEye, &vecAt );

#if (IMPLEMENTATION_STEP > 0)
	// Create our StreamTo and DrawFrom buffers
	const UINT MaxVertices = 300000;
	D3D10_BUFFER_DESC bd;
	bd.Usage = D3D10_USAGE_DEFAULT;
	bd.ByteWidth = MaxVertices * g_VertStride;
	bd.BindFlags = D3D10_BIND_VERTEX_BUFFER | D3D10_BIND_STREAM_OUTPUT;
	bd.CPUAccessFlags = 0;
	bd.MiscFlags = 0;
	V_RETURN( pd3dDevice->CreateBuffer( &bd, NULL, &g_pStreamTo ) );
	V_RETURN( pd3dDevice->CreateBuffer( &bd, NULL, &g_pDrawFrom ) );
#endif // (IMPLEMENTATION_STEP > 0)

#if (IMPLEMENTATION_STEP > 1)
	//-----------------------------------------------------------------------------------------
	// Create a buffer full of random vectors.  These will be indexed by our
	// tree generation algorithm to give the tree a more natural look.
	//-----------------------------------------------------------------------------------------
	int iNumRandValues = 1024;
	srand( 200 );
	//create the data
	D3D10_SUBRESOURCE_DATA InitData;
	InitData.pSysMem = new float[iNumRandValues * 4];
	if( !InitData.pSysMem )
		return E_OUTOFMEMORY;
	for( int i = 0; i < iNumRandValues * 4; i++ )
	{
		( ( float* )InitData.pSysMem )[i] = float( ( rand() % 10000 ) - 5000 ) / 5000.0f;
	}

	D3D10_BUFFER_DESC bufferDesc;
	bufferDesc.BindFlags = D3D10_BIND_SHADER_RESOURCE;
	bufferDesc.ByteWidth = 1024 * 4 * sizeof( float );
	bufferDesc.CPUAccessFlags = 0;
	bufferDesc.MiscFlags = 0;
	bufferDesc.Usage = D3D10_USAGE_DEFAULT;
	V_RETURN( pd3dDevice->CreateBuffer( &bufferDesc, &InitData, &g_pRandomBuffer ) );

	D3D10_SHADER_RESOURCE_VIEW_DESC SRVDesc;
	ZeroMemory( &SRVDesc, sizeof( SRVDesc ) );
	SRVDesc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
	SRVDesc.ViewDimension = D3D10_SRV_DIMENSION_BUFFER;
	SRVDesc.Buffer.ElementOffset = 0;
	SRVDesc.Buffer.ElementWidth = 1024;
	V_RETURN( pd3dDevice->CreateShaderResourceView( g_pRandomBuffer, &SRVDesc, &g_pRandomBufferRV ) );

	SAFE_DELETE_ARRAY( InitData.pSysMem );
#endif // (IMPLEMENTATION_STEP > 1)
	return S_OK;
}


//--------------------------------------------------------------------------------------
// Create any D3D10 resources that depend on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10SwapChainResized( ID3D10Device* pd3dDevice, IDXGISwapChain* pSwapChain,
										 const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	HRESULT hr = S_OK;

	// Setup the camera's projection parameters
	float fAspectRatio = pBackBufferSurfaceDesc->Width / ( FLOAT )pBackBufferSurfaceDesc->Height;
	g_Camera.SetProjParams( D3DX_PI / 4, fAspectRatio, 0.05f, 500.0f );
	g_Camera.SetWindow( pBackBufferSurfaceDesc->Width, pBackBufferSurfaceDesc->Height );
	g_Camera.SetButtonMasks( MOUSE_LEFT_BUTTON, MOUSE_WHEEL, MOUSE_MIDDLE_BUTTON );

	return hr;
}


#if (IMPLEMENTATION_STEP > 0)
void Smooth( ID3D10Device* pd3dDevice, int nPass )
{
	// Set IA parameters
	ID3D10Buffer* pBuffers[1];

	if( nPass == 0 )
	{
		pBuffers[0] = g_pVB;
	}
	else
	{
		pBuffers[0] = g_pDrawFrom;
	}

	UINT stride[1] = { g_VertStride };
	UINT offset[1] = { 0 };
	pd3dDevice->IASetInputLayout( g_pVertexLayout );
	pd3dDevice->IASetVertexBuffers( 0, 1, pBuffers, stride, offset );
	pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

	// Point to the correct output buffer
	pBuffers[0] = g_pStreamTo;
	pd3dDevice->SOSetTargets( 1, pBuffers, offset );

	// Draw
	D3D10_TECHNIQUE_DESC techDesc;
	g_pSmooth->GetDesc( &techDesc );

	for( UINT p = 0; p < techDesc.Passes; ++p )
	{
		g_pSmooth->GetPassByIndex( p )->Apply( 0 );
		if( nPass == 0 )
		{
			//If we're drawing from the original buffer (first time), draw the number of indices
			//that our mesh contains.
			pd3dDevice->Draw( NUM_VERTICES, 0 );
		}
		else
		{
			//If we're drawing from a buffer we've previously streamed to, let the hardware
			//determine how many triangles to draw.
			pd3dDevice->DrawAuto();
		}
	}

	// Get back to normal
	pBuffers[0] = NULL;
	pd3dDevice->SOSetTargets( 1, pBuffers, offset );

	// Swap stream out buffers
	ID3D10Buffer* pTemp = g_pDrawFrom;
	g_pDrawFrom = g_pStreamTo;
	g_pStreamTo = pTemp;
}
#endif // (IMPLEMENTATION_STEP > 0)

//--------------------------------------------------------------------------------------
// Render the scene using the D3D10 device
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
	// Clear the render target
	float ClearColor[4] = { 0.9569f, 0.9569f, 1.0f, 0.0f };
	ID3D10RenderTargetView* pRTV = DXUTGetD3D10RenderTargetView();
	pd3dDevice->ClearRenderTargetView( pRTV, ClearColor );
	ID3D10DepthStencilView* pDSV = DXUTGetD3D10DepthStencilView();
	pd3dDevice->ClearDepthStencilView( pDSV, D3D10_CLEAR_DEPTH, 1.0, 0 );

#if (IMPLEMENTATION_STEP > 0)
	// Make sure we only grow the tree the first time we go to render.
	// If we recreated the same tree every frame, we would definitely waste time.
	if( g_bGenerate )
	{
#if (IMPLEMENTATION_STEP > 1)
		g_pRandomBuf->SetResource( g_pRandomBufferRV );
#endif	//(IMPLEMENTATION_STEP > 1)	

		for( UINT i = 0; i < g_NumIterations; i++ )
			Smooth( pd3dDevice, i );

		g_bGenerate = false;
	}
#endif	//(IMPLEMENTATION_STEP > 0)	

	// Render the result
	D3DXMATRIX mWorld;
	D3DXMATRIX mView;
	D3DXMATRIX mProj;
	D3DXMATRIX mWorldView;
	D3DXMATRIX mWorldViewProj;
	mWorld = *g_Camera.GetWorldMatrix();
	mProj = *g_Camera.GetProjMatrix();
	mView = *g_Camera.GetViewMatrix();
	mWorldView = mWorld * mView;
	mWorldViewProj = mWorldView * mProj;

	// Set variables
	g_pmWorldViewProj->SetMatrix( ( float* )&mWorldViewProj );
	g_pmProj->SetMatrix( ( float* )&mProj );
	g_pmWorld->SetMatrix( ( float* )&mWorld );
	g_pmWorldView->SetMatrix( ( float* )&mWorldView );
	g_pDiffuseTex->SetResource( g_pMeshTexRV );

	D3DXVECTOR3 viewLightDir( 0,1,-1 );
	g_pViewSpaceLightDir->SetFloatVector( ( float* )&viewLightDir );

	// Set Input Assembler params
	ID3D10Buffer* pBuffers[1];

#if (IMPLEMENTATION_STEP > 0)	
	// NEW: MODIFIED VERTEX BUFFER
	if(g_NumIterations == 0) {
		pd3dDevice->IASetInputLayout( g_pVertexLayout );
		pBuffers[0] = g_pVB;
	} else {
		pBuffers[0] = g_pDrawFrom;
	}
#else //(IMPLEMENTATION_STEP > 0)
	pd3dDevice->IASetInputLayout( g_pVertexLayout );
	pBuffers[0] = g_pVB;
#endif	//(IMPLEMENTATION_STEP > 0)	

	UINT stride = g_VertStride;
	UINT offset = 0;
	pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
	pd3dDevice->IASetVertexBuffers( 0, 1, pBuffers, &stride, &offset );

	// Render using the technique g_pRenderTextured
	D3D10_TECHNIQUE_DESC techDesc;
	g_pRenderTextured->GetDesc( &techDesc );
	for( UINT p = 0; p < techDesc.Passes; p++ )
	{
		g_pRenderTextured->GetPassByIndex( p )->Apply( 0 );

#if (IMPLEMENTATION_STEP > 0)	
		if(g_NumIterations == 0) {
			pd3dDevice->Draw(NUM_VERTICES,0);
		} else {
			pd3dDevice->DrawAuto();
		}
#else //(IMPLEMENTATION_STEP > 0)	

		pd3dDevice->Draw(6,0);
#endif //(IMPLEMENTATION_STEP > 0)	
	}

	if (g_bShowNormals) {
		// SHOW NORMALS
		// Render using the technique g_pRenderTextured
		g_pShowNormals->GetDesc( &techDesc );
		for( UINT p = 0; p < techDesc.Passes; p++ )
		{
			g_pShowNormals->GetPassByIndex( p )->Apply( 0 );

#if (IMPLEMENTATION_STEP > 0)	
			if(g_NumIterations == 0) {
				pd3dDevice->Draw(NUM_VERTICES,0);
			} else {
				pd3dDevice->DrawAuto();
			}
#else //(IMPLEMENTATION_STEP > 0)	

			pd3dDevice->Draw(6,0);
#endif //(IMPLEMENTATION_STEP > 0)	

		}
	}
}


//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10CreateDevice 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10DestroyDevice( void* pUserContext )
{
	CDXUTDirectionWidget::StaticOnD3D10DestroyDevice();

	SAFE_RELEASE( g_pEffect10 );
	SAFE_RELEASE( g_pVertexLayout );
	SAFE_RELEASE( g_pVB );

	SAFE_RELEASE( g_pMeshTexRV );
#if (IMPLEMENTATION_STEP > 0)
	SAFE_RELEASE( g_pStreamTo );
	SAFE_RELEASE( g_pDrawFrom );
#endif

#if (IMPLEMENTATION_STEP > 1)
	SAFE_RELEASE( g_pRandomBufferRV);
	SAFE_RELEASE( g_pRandomBuffer);
#endif
}

//--------------------------------------------------------------------------------------
// Helper to load a VB and IB from a mesh 
//--------------------------------------------------------------------------------------
HRESULT LoadMesh( ID3D10Device* pd3dDevice )
{
	
	static float testVertexData[] =
	{
		//  --- position ---     --- normal ---      -texcoord- 
		0.0f, 0.0f,   1.5f,   0.0f, 1.0f, 2.0f,   0.5f, 1.0f,   
		1.0f, 1.5f,   0.0f,   1.0f, 1.0f, 0.0f,   1.0f, 0.0f,   
		-1.0f, 1.5f, -0.0f,   -1.0f, 1.0f, 0.0f,   0.0f, 0.0f,   

		1.0f, 1.5f,   0.0f,   1.0f, 1.0f, 0.0f,   1.0f, 0.0f,  
		0.0f, 0.0f,  -1.5f,   0.0f, 1.0f,-2.0f,   0.5f, 1.0f,  
		-1.0f, 1.5f, -0.0f,   -1.0f, 1.0f, 0.0f,   0.0f, 0.0f,  
	};

	HRESULT hr = S_OK;
	g_VertStride = sizeof( TRI_VERTEX );

	D3D10_BUFFER_DESC bd;
	bd.Usage = D3D10_USAGE_DEFAULT;
	bd.ByteWidth = sizeof( testVertexData );
	bd.BindFlags = D3D10_BIND_VERTEX_BUFFER;
	bd.CPUAccessFlags = 0;
	bd.MiscFlags = 0;
	D3D10_SUBRESOURCE_DATA InitData;
	InitData.pSysMem = testVertexData;
	V_RETURN( pd3dDevice->CreateBuffer( &bd, &InitData, &g_pVB ) );

	// Load the Texture
	WCHAR strPath[MAX_PATH] = {0};
	DXUTFindDXSDKMediaFileCch( strPath, sizeof( strPath ) / sizeof( WCHAR ), L"..\\media\\bark_diff.dds" );
	hr = D3DX10CreateShaderResourceViewFromFile( pd3dDevice, strPath, NULL, NULL, &g_pMeshTexRV, NULL );

	return hr;
}
