
//-----------------------------------------------------------------------------------------
// Input and Output Structures
//-----------------------------------------------------------------------------------------
struct VSSceneIn
{
	float3 Pos		: POS;
	float3 Norm		: NORMAL;
	float2 Tex		: TEXCOORD0;
};

struct GSSceneIn
{
	float3 Pos		: POS;
	float3 Norm		: NORMAL;
	float2 Tex		: TEXCOORD0;
};

struct PSSceneIn
{
	float4 Pos  : SV_Position;		// SV_Position is a (S)ystem (V)ariable that denotes transformed position
	float3 Norm : TEXCOORD0;		// World transformed normal
	float2 Tex  : TEXCOORD1;
};

//-----------------------------------------------------------------------------------------
// Constant Buffers (where we store variables by frequency of update)
//-----------------------------------------------------------------------------------------
cbuffer cbEveryFrame
{
	matrix g_mWorldViewProj;
	matrix g_mWorldView;
	matrix g_mWorld;
	matrix g_mProj;
	float3 g_ViewSpaceLightDir;
};

cbuffer cbUserDefined
{
	float LengthModifier = 0.9;
	float SpreadModifier = 3;
	float ShrinkModifier = 0.5;
};

//-----------------------------------------------------------------------------------------
// Textures and Samplers
//-----------------------------------------------------------------------------------------
Texture2D g_txDiffuse;
sampler2D g_samLinear = sampler_state
{
	texture=g_txDiffuse;
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = WRAP;
	AddressV = WRAP;
};


//-----------------------------------------------------------------------------------------
// State Structures
//-----------------------------------------------------------------------------------------
BlendState NoBlending
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = FALSE;
};

DepthStencilState DisableDepth
{
    DepthEnable = FALSE;
    DepthWriteMask = ZERO;
};

DepthStencilState EnableDepth
{
    DepthEnable = TRUE;
    DepthWriteMask = ALL;
};

RasterizerState NoCull
{
	CullMode = None;
};

//-----------------------------------------------------------------------------------------
// VertexShader: VSScene
//-----------------------------------------------------------------------------------------
GSSceneIn VSScene(VSSceneIn input)
{
	GSSceneIn output;
	
	// Transform the Point
	output.Pos = mul( float4(input.Pos,1.0), g_mWorldView );
	
	// Transform the Normal
	output.Norm = normalize(mul( input.Norm, (float3x3)g_mWorldView ));
	
	// Pass the texcoord through
	output.Tex = input.Tex;
		
	return output;
}

//-----------------------------------------------------------------------------------------
// PixelShader: PSSceneMain
//-----------------------------------------------------------------------------------------
float4 PSScene(PSSceneIn input) : SV_Target
{	
	//calculate the lighting
	float lighting = saturate( dot( normalize( input.Norm ), g_ViewSpaceLightDir ) );
	return tex2D( g_samLinear, input.Tex)*lighting;
}

//-----------------------------------------------------------------------------------------
// PixelShader: PSSceneMain
//-----------------------------------------------------------------------------------------
float4 PSShowNormals(PSSceneIn input) : SV_Target
{	
	//calculate the ligh
	return float4(1.0,0.0,0.0,1.0);
}

//-----------------------------------------------------------------------------------------
// Helper Functions
//-----------------------------------------------------------------------------------------

float3 GetNormal( float3 A, float3 B, float3 C )
{
	float3 AB = B - A;
	float3 AC = C - A;
	return normalize( cross(AB,AC) );
}

//-----------------------------------------------------------------------------------------
// GeometryShader: GSGrowBranches
//					Grow a branch off of any triangles that have a non-zero GrowAmounts.
//					Maximimum number of vertices that can be output from this geometry
//					shader is 17.
//-----------------------------------------------------------------------------------------
[maxvertexcount(8)]
void GSScene( triangle GSSceneIn input[3], inout TriangleStream<PSSceneIn> OutputStream )
{
	PSSceneIn inner[3];
	PSSceneIn orig[3];
	
	[unroll] for( uint i=0; i<3; i++ )
	{
		orig[i].Pos   = mul(float4(input[i].Pos.xyz,1), g_mProj );
		orig[i].Norm = input[i].Norm;
		orig[i].Tex  = input[i].Tex;
	
		uint iNext = (i+1)%3;
		
		float3 P0 = input[i].Pos;
		float3 P1 = input[iNext].Pos;
		
		float3 d  = P0 - P1;	
		float3 N = 	(input[i].Norm + input[iNext].Norm) / 2;
		float3 b0 = cross(N,d); // steht senkrect auf Ebene
		float3 N0 = input[i].Norm - dot(input[i].Norm,b0)*b0;
		float3 N1 = input[iNext].Norm - dot(input[iNext].Norm,b0)*b0;
		float3 d0 = cross(N0,b0);
		float3 d1 =-cross(N1,b0);
		
		float m1 = ( (P1.y  - P0.y) * d0.x + (P0.x - P1.x) * d0.y ) / ( d1.x * d0.y - d1.y * d0.x);
		float m0 = ( (P0.y  - P1.y) * d1.x + (P1.x - P0.x) * d1.y ) / ( d0.x * d1.y - d0.y * d1.x);
		
		float3 PxA = input[iNext].Pos + m1 * d1;
		float3 PxB = input[i].Pos + m0 * d0;
		float3 Px = (PxA + PxB)/2.0;
		float3 Pa = (P0+Px)/2.0;
		float3 Pb = (P1+Px)/2.0;
		float3 Pnew = (Pa+Pb)/2.0;
		float interp = length(P0-Pnew)/(length(P0-Pnew)+length(P1-Pnew));
	
		inner[i].Norm  = normalize(lerp(input[iNext].Norm,input[i].Norm,interp));
		inner[i].Tex   = lerp(input[iNext].Tex, input[i].Tex, interp);
		inner[i].Pos   = mul(float4(Pnew,1), g_mProj );
	}
	
	OutputStream.Append(orig[0]);
	OutputStream.Append(inner[0]);
	OutputStream.Append(inner[2]);
	OutputStream.Append(inner[1]);
	OutputStream.Append(orig[2]);		
	OutputStream.RestartStrip();
	
	OutputStream.Append(inner[0]);
	OutputStream.Append(orig[1]);
	OutputStream.Append(inner[1]);		
	OutputStream.RestartStrip();
	
}

//-----------------------------------------------------------------------------------------
// GeometryShader: GSGrowBranches
//					Grow a branch off of any triangles that have a non-zero GrowAmounts.
//					Maximimum number of vertices that can be output from this geometry
//					shader is 17.
//-----------------------------------------------------------------------------------------
[maxvertexcount(6)]
void GSShowNormals( triangle GSSceneIn input[3], inout LineStream<PSSceneIn> OutputStream  )
{
	PSSceneIn output;
	for( uint i=0; i<3; i++ )
	{
		output.Pos = mul(float4(input[i].Pos,1), g_mProj );
		output.Norm = input[i].Norm;
		output.Tex = input[i].Tex;
		OutputStream.Append( output );
		
		output.Pos = mul(float4(input[i].Pos + input[i].Norm,1), g_mProj );
		output.Norm = input[i].Norm;
		output.Tex = input[i].Tex;
		OutputStream.Append( output );
		OutputStream.RestartStrip();
	}
	
}

//-----------------------------------------------------------------------------------------
// Technique: RenderTextured  
//-----------------------------------------------------------------------------------------
technique10 RenderTextured
{
    pass p0
    {
		// Set VS, GS, and PS
        SetVertexShader( CompileShader( vs_4_0, VSScene() ) );
        SetGeometryShader( CompileShader( gs_4_0, GSScene() ) );
        SetPixelShader( CompileShader( ps_4_0, PSScene() ) );
        
        SetRasterizerState(NoCull);
        SetDepthStencilState( EnableDepth, 0 );
    }  
}

//-----------------------------------------------------------------------------------------
// Technique: RenderTextured  
//-----------------------------------------------------------------------------------------
technique10 ShowNormals
{
    pass p0
    {
		// Set VS, GS, and PS
        SetVertexShader( CompileShader( vs_4_0, VSScene() ) );
        SetGeometryShader( CompileShader( gs_4_0, GSShowNormals() ) );
        SetPixelShader( CompileShader( ps_4_0, PSShowNormals() ) );
        
        SetRasterizerState(NoCull);
        SetDepthStencilState( EnableDepth, 0 );
    }  
}
